# Copyright (C) 2003 Peter J. Verveer
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met: 
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#
# 3. The name of the author may not be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.      

import types
import numarray

def _extend_mode_to_code(mode):
    """Convert an extension mode to the corresponding integer code.
    """
    if mode == 'nearest':
        return 0
    elif mode == 'wrap':
        return 1
    elif mode == 'reflect':
        return 2
    elif mode == 'constant':
        return 4
    else:
        raise RuntimeError, 'boundary mode not supported'

def _normalize_sequence(input, array):
    """If input is a scalar, create a sequence of length equal to the
    rank of array by duplicating the input. If input is a sequence,
    check if its length is equalt to the lenght of array.

    """
    if isinstance(array, numarray.NumArray):
        rank = len(array.shape)
    else:
        rank = 1
    if (isinstance(input, (types.IntType, types.LongType, types.FloatType))):
        normalized = [input] * rank
    else:
        if isinstance(input, numarray.NumArray):
            normalized = list(input)
        else:
            normalized = input
        if len(normalized) != rank:
            error = "sequence argument must have length equal to input rank"
            raise RuntimeError, error
    return normalized


def _array_rank(array):
    """Return the rank of the input, if it is an array. Return 1 if the input
    is a sequence. Return 0 otherwise."""
    if isinstance(array, numarray.NumArray):
        return len(array.shape)
    elif (isinstance(array, (types.TupleType, types.ListType))):
        return 1
    else:
        return 0
    
